use Encode;
use CGI;
use HTML::Entities;
use Config::IniFiles;
use CLI::Template;
use File::Basename;
use lib dirname (__FILE__);
use Scalar::Util qw(looks_like_number);
require "properties.pl";
our $cnPkgDir;
our $customPkgDir;
our $readOnlyMode;

our $cli;

sub init {
    my $obj = shift;
    $cli = $obj;

    # Copy bits to CLI space
    $cli->self->env->{ cnReportPath } = $cnPkgDir;
    $cli->self->env->{ reportPath } = $customPkgDir;

    if ( -f "/tmp/reports.debug" ) {
        $obj->self->env->{ CTF_DEBUG   } = 1;
        $obj->self->env->{ DEBUG_DEPTH } = 500;
    }
}

sub loadConfig {
    my($cfg,%ini);

    my $pkg    = shift;
    my $box    = $cli->self->env->{ box };
    my $pkgdir = sprintf("%s/%s", $customPkgDir, basename($pkg));

    unless ( -d $pkgdir ) {
        $pkgdir = sprintf("%s/%s", $cnPkgDir, basename($pkg));
    }

    my $file   = "$pkgdir/config.ini";

    if ( -f $file ) {
        tie %ini, 'Config::IniFiles', ( -file => $file );
        $cfg = {
            cfg    => \%ini,
            script => "$pkgdir/script",
            types  => { map { $_, 1 } split(/,/, $ini{main}{outputType})}
        };
    } else {
        print STDERR "[report-libs::loadConfig] file not found.\n";
    }
    $cli->self->env->{'report'} = $cfg;
    $cli->self->env->{'configini'} = \%ini;

    for my $key (split(/,/, $ini{main}{fields})) {
        $box->{ field }->{ $key } ||= $ini{$key}{default};
    }

    for my $key (keys( %{ $ini{ $box->{ chart } } })) {
        $box->{ props }->{ $key } ||= $box->{ chart }{ $key };
    }

    return "";
}

sub convertSnvToHash {
    my $snv = shift;
    my %h;
    @h{@{$snv->{'names'}}} = @{$snv->{'values'}};
    return \%h;
}

# Expected fields in properties should look like this:
#
#    box1.field.fieldName
#    box1.props.pName

sub readProps {
    my $obj    = shift;
    my $data   = convertSnvToHash($obj->data->{'settings'});
    my $pcId   = $obj->self->env->{'sfPagecId'};
    my $boxId  = $obj->self->env->{'sfBoxId'};

    my $box = {
	id        => $boxId,
	pagecId   => $pcId,
        report    => $data->{ sprintf("%s.report", $boxId) },
        chart     => $data->{ sprintf("%s.chart", $boxId) },
        showSampleReport    => $data->{ sprintf("%s.showSampleReport", $boxId) },
        props     => {},
        field     => {},
    };
    $obj->self->env->{'box'} = $box;
    
    for my $field (grep(/^$boxId\./, keys( %{ $data } ) )) {
        my @p = split(/\./, $field );
        $data->{ $field } =~ s/^\s+//;
        $data->{ $field } =~ s/\s+$//;
        $box->{ $p[1] }->{ $p[2] } = $data->{ $field };
    }
    if (not $obj->self->env->{'noOfOpenTabs'} ~~ [1..3]) {
        foreach my $i (reverse(1..3)) {
            $obj->self->env->{'noOfOpenTabs'} = $i;
            last if (exists $data->{sprintf('box%s.report', $i)} &&
                        $data->{sprintf('box%s.report', $i)} ne 'none');
        }
    }
}

sub initProps {
    my $obj    = shift;
    my $pcId   = $obj->self->env->{'sfPagecId'};
    my $boxId  = $obj->self->env->{'sfBoxId'};

    my $box = {
        id        => $boxId,
        pagecId   => $pcId,
        report    => '',
        chart     => '',
        props     => {},
        field     => {},
    };
    $obj->self->env->{'box'} = $box;
}

sub getBoxProps {
    my $obj = shift;
    my $hash = shift || {};
    my $box = $obj->self->env->{ box };

    for my $type (qw!report chart!) {
        my $hkey = sprintf("%s.%s", $box->{ id }, $type );
        $hash->{ $hkey } = $box->{ $type };
    }
    
    my $hkey = sprintf("%s.showSampleReport", $box->{ id });
    $hash->{ $hkey } = $box->{ showSampleReport };

    for my $type (qw!field props!) {
        for my $key (keys(%{ $box->{ $type } })) {
            my $hkey = sprintf("%s.%s.%s", $box->{ id }, $type, $key );
            $hash->{ $hkey } = $box->{ $type }->{ $key };
        }
    }
    return $hash;
}

sub decodeProp {
    my $val = shift;
    my $isNumeric = shift;
    if (looks_like_number($val)) {
      return $val;
    } else {
      if ($isNumeric) {
         return 0;
      }
      my $boxval = $val;
      $$boxval = $boxval;
      $$boxval = HTML::Entities::encode_entities($$boxval);
      $$boxval = CGI::escapeHTML($$boxval);
      return $$boxval;
    }
}

sub updateBoxProperty {
    my $obj = shift;
    my $key = shift;
    my $val = shift;
    my $boxval = \$obj->self->env->{ box }->{ props }->{ $key };
    $$boxval = $val || $$boxval;

    #Decode the utf8 params and escape the html.
    $$boxval = Encode::decode_utf8($$boxval);
    $$boxval = escapeHTML($$boxval);
}

sub saveBox {
    my $obj = shift;
    my $content = shift || $obj->data->{'readOnlyTextSource'};
    my $pagec = $obj->data;
    my $hash = convertSnvToHash($pagec->{'settings'});
    $hash = getBoxProps($obj, $hash);
    updatePageComponent($obj, $hash, $content);
}

sub delBox {
    my $obj = shift;
    my $env = $obj->self->env;
    my $pagec = $obj->data;
    my $content = $env->{'CONTENT'};
    my $box = $obj->self->env->{ box };
    my $hash = convertSnvToHash($pagec->{'settings'});

    delete $hash->{ sprintf("%s.%s", $box->{ id }, 'chart') };
    $hash->{ sprintf("%s.%s", $box->{ id }, 'report') } = 'none';
    delete $hash->{ sprintf("%s.%s", $box->{ id }, 'showSampleReport') };

    for my $type (qw!field props!) {
        for my $key (keys(%{ $box->{ $type } })) {
            delete $hash->{ sprintf("%s.%s.%s", $box->{ id }, $type, $key ) };
        }
    }
    updatePageComponent($obj, $hash, $content);
}

# To update only content, call updatePageComponent($obj, undef, $newContent);
sub updatePageComponent {
    my $obj = shift;
    my $hash = shift;
    my $content = shift || $obj->data->{'readOnlyTextSource'};
    my $pagec = $obj->data;
    my $env = $obj->self->env;

    if (not defined $hash) {
        $hash = convertSnvToHash($pagec->{'settings'});
    }

    if ($env->{'is_template'} eq 'true') {
        $hash->{sprintf('%s.showSampleReport', $env->{'sfBoxId'})} = 'true';
    }

    $pagec->{'settings'} = SoapNamedValues->new($hash);
    $obj->self->invokeMethod('setLifecycleMetricComponentContent', {
        pageComponent => $pagec,
        content => $content
    }, 'PageApp');
}

sub initChart {
    my $type   = shift;    
    my $box    = $cli->self->env->{ box };
    my $report = $cli->self->env->{ report };

    $box->{ props } = {};
    $box->{ chart } = $type;

    if ( $report->{ cfg }->{ $type } ) {
	for my $key (keys(%{ $report->{ cfg }->{ $type } })) {
	    $box->{ props }->{ $key } = $report->{ cfg }->{ $type }->{ $key };
	}
    }
}

sub updateTitle {
    my $obj = shift;
    my $ref = $obj->self->env->{"box"}->{"props"};
    if ( $ref->{"title"} ) {
	my $ans = CLI::Template->simple($obj, $ref->{"title"} )->handle( $obj, 'simple');
	$ref->{"title"} = $ans;
    }
}

sub fixContent {
    my $obj = shift;
    my $env = $obj->self->env;
    my $box = $env->{ box }->{ id };
    $env->{ CONTENT } =~ s/<\!-- START BOX $box .* END BOX $box -->/$env->{ BOX }/s;
    $env->{ CONTENT } =~ s/\n+/\n/mg;
}

sub getTypeNameHash {
    return {
        'artf' => 'Artifact',
        'tracker' => 'Tracker',
        'forum' => 'Forum',
        'topc' => 'Topic',
        'post' => 'Post',
        'news' => 'NewsPost',
        'plan' => 'Plan',
        'wiki' => 'Wiki',
        'pkg' => 'Package',
        'rel' => 'Release',
        'frs' => 'FrsFile',
        'reps' => 'Repository',
        'cmmt' => 'Commit',
        'docf' => 'DocumentFolder',
        'doc' => 'Document',
        'docr' => 'DocumentReview',
        'user' => 'User',
        'page' => 'Page',
        'task' => 'Task',
        'taskgrp' => 'TaskGroup',
    };

sub escapeHTML {
     my ( $escape, $text ) = @_;
     return undef unless defined $escape;
         $escape =~ s/&/\\&/g;
         $escape =~ s/"/\\"/g;
         $escape =~ s/'/\\'/g;
         $escape =~ s/</&lt;/g;
         $escape =~ s/>/&gt;/g;         
         # these next optional escapes make text look the same when rendered in HTML
         if ( $text ) {
             $escape =~ s/\t/    /g;                          # tabs to 4 spaces
             $escape =~ s/( {2,})/"&nbsp;" x length $1/eg;    # whitespace escapes
             $escape =~ s/\n/<br>\n/g;                        # newlines to <br>
         }
      return $escape;
     }
}

1;
